@@ -13,7 +13,6 @@ my $testfh;
 if (exists $ENV{TEST_OUTPUT}) {
 	my $file = $ENV{TEST_OUTPUT};
 	open $testfh, '>>', $file or die qq{Could not append file "$file": $!\n};
-	#Test::More->builder->output($testfh);
 	Test::More->builder->failure_output($testfh);
 	Test::More->builder->todo_output($testfh);
 }
@@ -68,6 +67,7 @@ sub connect_database {
 	my $alias = qr{(database|db|dbname)};
 	my $info;
 	my $olddir = getcwd;
+	my $debug = $ENV{DBDPG_DEBUG} || 0;
 
 	## We'll try various ways to get to a database to test with
 
@@ -75,6 +75,21 @@ sub connect_database {
 	my ($testdsn,$testuser,$helpconnect,$su,$uid,$testdir,$pg_ctl,$initdb,$error,$version)
 		= get_test_settings();
 
+	if ($debug) {
+		diag "Test settings:
+dsn: $testdsn
+user: $testuser
+helpconnect: $helpconnect
+su: $su
+uid: $uid
+testdir: $testdir
+pg_ctl: $pg_ctl
+initdb: $initdb
+error: $error
+version: $version
+";
+	}
+
 	## Did we fail last time? Fail this time too, but quicker!
 	if ($testdsn =~ /FAIL!/) {
 		return $helpconnect, "Previous failure ($error)", undef;
@@ -102,7 +117,7 @@ sub connect_database {
 			1;
 		};
 
-		if ($@ =~ /invalid connection option/) {
+		if ($@ =~ /invalid connection option/ or $@ =~ /failed:.*"dbbarf"/) {
 			return $helpconnect, $@, undef;
 		}
 
@@ -195,7 +210,7 @@ sub connect_database {
 
 	if (! $testdsn) {
 		$helpconnect = 1;
-		$testdsn = $^O =~ /Win32/ ? 'dbi:Pg:host=localhost' : 'dbi:Pg:';
+#		$testdsn = $^O =~ /Win32/ ? 'dbi:Pg:host=localhost' : 'dbi:Pg:';
 	}
 	if (! $testuser) {
 		$testuser = 'postgres';
@@ -208,7 +223,6 @@ sub connect_database {
 								{RaiseError => 1, PrintError => 0, AutoCommit => 1});
 		};
 		last GETHANDLE if ! $@; ## Made it!
-
 		## If the error was because of the user, try a few others
 		if ($@ =~ /postgres/) {
 
@@ -380,6 +394,20 @@ sub connect_database {
 			last GETHANDLE; ## Fail - bad output
 		}
 
+		## Attempt to boost the system oids above an int for certain testing
+		(my $resetxlog = $initdb) =~ s/initdb/pg_resetxlog/;
+		eval {
+			$info = qx{$resetxlog --help};
+		};
+		if (! $@ and $info =~ /XID/) {
+			if (! -e "$testdir/data/postmaster.pid") {
+				eval {
+					$info = qx{ $resetxlog -o 2222333344 $testdir/data };
+				};
+				## We don't really care if it worked or not!
+			}
+		}
+
 		## Which user do we connect as?
 		if (!$su and $info =~ /owned by user "(.+?)"/) {
 			$testuser = $1;
@@ -411,40 +439,60 @@ sub connect_database {
 			}
 		}
 		$@ = '';
-		## Change to this new port and fire it up
+
+		$debug and diag "Port to use: $testport";
+
 		my $conf = "$testdir/data/postgresql.conf";
 		my $cfh;
-		if (! open $cfh, '>>', $conf) {
-			$@ = qq{Could not open "$conf": $!};
-			last GETHANDLE; ## Fail - no conf file
-		}
-		print $cfh "\n\n## DBD::Pg testing parameters\n";
-		print $cfh "port=$testport\n";
-		print $cfh "max_connections=4\n";
-		if ($version >= 8.0) {
-			print $cfh "log_statement = 'all'\n";
-			print $cfh "log_line_prefix = '%m [%p] '\n";
-		}
-		else {
-			print $cfh "silent_mode = true\n";
-		}
-		if ($version == 8.1) {
-			print {$cfh} "redirect_stderr = on\n";
-		}
 
-		if ($version >= 8.3) {
-			print {$cfh} "logging_collector = on\n";
-		}
-		print $cfh "log_min_messages = 'DEBUG1'\n";
-		print $cfh "listen_addresses='127.0.0.1'\n" if $^O =~ /Win32/;
-		print $cfh "\n";
-		close $cfh or die qq{Could not close "$conf": $!\n};
-
-		## Attempt to start up the test server
+		## If there is already a pid file, do not modify the config
+		## We assume a previous run put it there, so we extract the port
 		if (-e "$testdir/data/postmaster.pid") {
+			$debug and diag qq{File "$testdir/data/postmaster.pid" exists};
+			open my $cfh, '<', $conf or die qq{Could not open "$conf": $!\n};
+			while (<$cfh>) {
+				if (/^\s*port\s*=\s*(\d+)/) {
+					$testport = $1;
+					$debug and diag qq{Found port $testport inside conf file\n};
+				}
+			}
+			close $cfh or die qq{Could not close "$conf": $!\n};
 			## Assume it's up, and move on
 		}
 		else {
+			## Change to this new port and fire it up
+			if (! open $cfh, '>>', $conf) {
+				$@ = qq{Could not open "$conf": $!};
+				$debug and diag qq{Failed to open "$conf"};
+				last GETHANDLE; ## Fail - no conf file
+			}
+			$debug and diag qq{Writing to "$conf"};
+			print $cfh "\n\n## DBD::Pg testing parameters\n";
+			print $cfh "port=$testport\n";
+			print $cfh "max_connections=5\n";
+			if ($version >= 8.0) {
+				print $cfh "log_statement = 'all'\n";
+				print $cfh "log_line_prefix = '%m [%p] '\n";
+			}
+			else {
+				print $cfh "silent_mode = true\n";
+			}
+			if ($version == 8.1) {
+				print {$cfh} "redirect_stderr = on\n";
+			}
+
+			if ($version >= 8.3) {
+				print {$cfh} "logging_collector = on\n";
+			}
+			print $cfh "log_min_messages = 'DEBUG1'\n";
+			print $cfh "log_filename = 'postgres%Y-%m-%d.log'\n";
+			print $cfh "log_rotation_size = 0\n";
+
+			print $cfh "listen_addresses='127.0.0.1'\n" if $^O =~ /Win32/;
+			print $cfh "\n";
+			close $cfh or die qq{Could not close "$conf": $!\n};
+
+			## Attempt to start up the test server
 			$info = '';
 			my $option = '';
 			if ($^O !~ /Win32/) {
@@ -468,6 +516,7 @@ sub connect_database {
 				chdir $testdir;
 				$COM = qq{su -m $su -c "$COM"};
 			}
+			$debug and diag qq{Running: $COM};
 			eval {
 				$info = qx{$COM};
 			};
@@ -489,6 +538,7 @@ sub connect_database {
 			$testdsn .= ";host=$testdir/data/socket";
 		}
 
+		$debug and diag qq{Test DSN: $testdsn};
 		my $loop = 1;
 	  STARTUP: {
 			eval {
@@ -497,8 +547,10 @@ sub connect_database {
 			};
 			## Regardless of the error, try again.
 			## We used to check the message, but LANG problems may complicate that.
-
 			if ($@) {
+
+				$debug and diag qq{Connection error: $@\n};
+
 				if ($@ =~ /database "postgres" does not exist/) {
 					## Old server, so let's create a postgres database manually
 					sleep 2;
@@ -507,12 +559,29 @@ sub connect_database {
 						$dbh = DBI->connect($tempdsn, $testuser, '',
 											{RaiseError => 1, PrintError => 0, AutoCommit => 1});
 					};
-					$dbh->do('CREATE DATABASE postgres');
-					$dbh->disconnect();
 					if ($@) {
 						die "Could not connect: $@\n";
 					}
+					$dbh->do('CREATE DATABASE postgres');
+					$dbh->disconnect();
 				}
+
+				if ($@ =~ /role "postgres" does not exist/) {
+					## Probably just created with the current user, so use that
+					if (exists $ENV{USER} and length $ENV{USER}) {
+						$debug and diag qq{Switched to new user: $testuser\n};
+						eval {
+							$dbh = DBI->connect($testdsn, $ENV{USER}, '',
+												{RaiseError => 1, PrintError => 0, AutoCommit => 1});
+						};
+						if ($@) {
+							die "Could not connect: $@\n";
+						}
+						$dbh->do('CREATE USER postgres SUPERUSER');
+						$dbh->disconnect();
+					}
+				}
+
 				if ($loop++ < 5) {
 					sleep 1;
 					redo STARTUP;
